@@ -30,12 +30,14 @@ import (
 	"strings"
 
 	"github.com/AlecAivazis/survey/v2"
+	"github.com/AlecAivazis/survey/v2/terminal"
+	"github.com/MakeNowJust/heredoc/v2"
 	"github.com/apex/log"
 	"github.com/blacktop/ipsw/internal/commands/extract"
-	"github.com/blacktop/ipsw/internal/commands/img4"
 	"github.com/blacktop/ipsw/internal/download"
 	"github.com/blacktop/ipsw/internal/utils"
 	"github.com/blacktop/ipsw/pkg/dyld"
+	"github.com/blacktop/ipsw/pkg/img4"
 	"github.com/blacktop/ipsw/pkg/info"
 	"github.com/fatih/color"
 	"github.com/pkg/errors"
@@ -44,55 +46,94 @@ import (
 )
 
 func init() {
-	DownloadCmd.AddCommand(ipswCmd)
-
-	ipswCmd.Flags().Bool("latest", false, "Download latest IPSWs")
-	ipswCmd.Flags().Bool("show-latest-version", false, "Show latest iOS version")
-	ipswCmd.Flags().Bool("show-latest-build", false, "Show latest iOS build")
-	ipswCmd.Flags().Bool("macos", false, "Download macOS IPSWs")
-	ipswCmd.Flags().Bool("ibridge", false, "Download iBridge IPSWs")
-	ipswCmd.Flags().Bool("kernel", false, "Extract kernelcache from remote IPSW")
-	ipswCmd.Flags().Bool("dyld", false, "Extract dyld_shared_cache(s) from remote IPSW")
-	ipswCmd.Flags().StringArrayP("dyld-arch", "a", []string{}, "dyld_shared_cache architecture(s) to remote extract")
-	ipswCmd.RegisterFlagCompletionFunc("dyld-arch", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	DownloadCmd.AddCommand(downloadIpswCmd)
+	// Download behavior flags
+	downloadIpswCmd.Flags().String("proxy", "", "HTTP/HTTPS proxy")
+	downloadIpswCmd.Flags().Bool("insecure", false, "do not verify ssl certs")
+	downloadIpswCmd.Flags().BoolP("confirm", "y", false, "do not prompt user for confirmation")
+	downloadIpswCmd.Flags().Bool("skip-all", false, "always skip resumable IPSWs")
+	downloadIpswCmd.Flags().Bool("resume-all", false, "always resume resumable IPSWs")
+	downloadIpswCmd.Flags().Bool("restart-all", false, "always restart resumable IPSWs")
+	downloadIpswCmd.Flags().BoolP("remove-commas", "_", false, "replace commas in IPSW filename with underscores")
+	// Filter flags
+	downloadIpswCmd.Flags().StringArray("white-list", []string{}, "iOS device white list")
+	downloadIpswCmd.Flags().StringArray("black-list", []string{}, "iOS device black list")
+	downloadIpswCmd.Flags().StringP("device", "d", "", "iOS Device (i.e. iPhone11,2)")
+	downloadIpswCmd.Flags().StringP("model", "m", "", "iOS Model (i.e. D321AP)")
+	downloadIpswCmd.Flags().StringP("version", "v", "", "iOS Version (i.e. 12.3.1)")
+	downloadIpswCmd.Flags().StringP("build", "b", "", "iOS BuildID (i.e. 16F203)")
+	// IPSW-specific flags
+	downloadIpswCmd.Flags().Bool("latest", false, "Download latest IPSWs")
+	downloadIpswCmd.Flags().Bool("show-latest-version", false, "Show latest iOS version")
+	downloadIpswCmd.Flags().Bool("show-latest-build", false, "Show latest iOS build")
+	downloadIpswCmd.Flags().Bool("macos", false, "Download macOS IPSWs")
+	downloadIpswCmd.Flags().Bool("ibridge", false, "Download iBridge IPSWs")
+	downloadIpswCmd.Flags().Bool("kernel", false, "Extract kernelcache from remote IPSW")
+	downloadIpswCmd.Flags().Bool("dyld", false, "Extract dyld_shared_cache(s) from remote IPSW")
+	downloadIpswCmd.Flags().StringArrayP("dyld-arch", "a", []string{}, "dyld_shared_cache architecture(s) to remote extract")
+	downloadIpswCmd.RegisterFlagCompletionFunc("dyld-arch", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return dyld.DscArches, cobra.ShellCompDirectiveDefault
 	})
-	// ipswCmd.Flags().BoolP("kernel-spec", "", false, "Download kernels into spec folders")
-	ipswCmd.Flags().String("pattern", "", "Download remote files that match regex")
-	ipswCmd.Flags().Bool("fcs-keys", false, "Download AEA1 DMG fcs-key pem files")
-	ipswCmd.Flags().Bool("fcs-keys-json", false, "Download AEA1 DMG fcs-keys as JSON")
-	ipswCmd.Flags().Bool("decrypt", false, "Attempt to decrypt the partial files if keys are available")
-	ipswCmd.Flags().BoolP("flat", "f", false, "Do NOT perserve directory structure when downloading with --pattern")
-	ipswCmd.Flags().BoolP("urls", "u", false, "Dump URLs only")
-	ipswCmd.Flags().Bool("usb", false, "Download IPSWs for USB attached iDevices")
-	ipswCmd.Flags().StringP("output", "o", "", "Folder to download files to")
-	ipswCmd.MarkFlagDirname("output")
-
-	viper.BindPFlag("download.ipsw.latest", ipswCmd.Flags().Lookup("latest"))
-	viper.BindPFlag("download.ipsw.show-latest-version", ipswCmd.Flags().Lookup("show-latest-version"))
-	viper.BindPFlag("download.ipsw.show-latest-build", ipswCmd.Flags().Lookup("show-latest-build"))
-	viper.BindPFlag("download.ipsw.macos", ipswCmd.Flags().Lookup("macos"))
-	viper.BindPFlag("download.ipsw.ibridge", ipswCmd.Flags().Lookup("ibridge"))
-	viper.BindPFlag("download.ipsw.kernel", ipswCmd.Flags().Lookup("kernel"))
-	viper.BindPFlag("download.ipsw.dyld", ipswCmd.Flags().Lookup("dyld"))
-	viper.BindPFlag("download.ipsw.dyld-arch", ipswCmd.Flags().Lookup("dyld-arch"))
-	// viper.BindPFlag("download.ipsw.kernel-spec", ipswCmd.Flags().Lookup("kernel-spec"))
-	viper.BindPFlag("download.ipsw.pattern", ipswCmd.Flags().Lookup("pattern"))
-	viper.BindPFlag("download.ipsw.fcs-keys", ipswCmd.Flags().Lookup("fcs-keys"))
-	viper.BindPFlag("download.ipsw.fcs-keys-json", ipswCmd.Flags().Lookup("fcs-keys-json"))
-	viper.BindPFlag("download.ipsw.decrypt", ipswCmd.Flags().Lookup("decrypt"))
-	viper.BindPFlag("download.ipsw.output", ipswCmd.Flags().Lookup("output"))
-	viper.BindPFlag("download.ipsw.flat", ipswCmd.Flags().Lookup("flat"))
-	viper.BindPFlag("download.ipsw.urls", ipswCmd.Flags().Lookup("urls"))
-	viper.BindPFlag("download.ipsw.usb", ipswCmd.Flags().Lookup("usb"))
+	// downloadIpswCmd.Flags().BoolP("kernel-spec", "", false, "Download kernels into spec folders")
+	downloadIpswCmd.Flags().String("pattern", "", "Download remote files that match regex")
+	downloadIpswCmd.Flags().Bool("fcs-keys", false, "Download AEA1 DMG fcs-key pem files")
+	downloadIpswCmd.Flags().Bool("fcs-keys-json", false, "Download AEA1 DMG fcs-keys as JSON")
+	downloadIpswCmd.Flags().Bool("decrypt", false, "Attempt to decrypt the partial files if keys are available")
+	downloadIpswCmd.Flags().BoolP("flat", "f", false, "Do NOT perserve directory structure when downloading with --pattern")
+	downloadIpswCmd.Flags().BoolP("urls", "u", false, "Dump URLs only")
+	downloadIpswCmd.Flags().Bool("usb", false, "Download IPSWs for USB attached iDevices")
+	downloadIpswCmd.Flags().StringP("output", "o", "", "Folder to download files to")
+	downloadIpswCmd.MarkFlagDirname("output")
+	// Bind download behavior flags
+	viper.BindPFlag("download.ipsw.proxy", downloadIpswCmd.Flags().Lookup("proxy"))
+	viper.BindPFlag("download.ipsw.insecure", downloadIpswCmd.Flags().Lookup("insecure"))
+	viper.BindPFlag("download.ipsw.confirm", downloadIpswCmd.Flags().Lookup("confirm"))
+	viper.BindPFlag("download.ipsw.skip-all", downloadIpswCmd.Flags().Lookup("skip-all"))
+	viper.BindPFlag("download.ipsw.resume-all", downloadIpswCmd.Flags().Lookup("resume-all"))
+	viper.BindPFlag("download.ipsw.restart-all", downloadIpswCmd.Flags().Lookup("restart-all"))
+	viper.BindPFlag("download.ipsw.remove-commas", downloadIpswCmd.Flags().Lookup("remove-commas"))
+	// Bind filter flags
+	viper.BindPFlag("download.ipsw.white-list", downloadIpswCmd.Flags().Lookup("white-list"))
+	viper.BindPFlag("download.ipsw.black-list", downloadIpswCmd.Flags().Lookup("black-list"))
+	viper.BindPFlag("download.ipsw.device", downloadIpswCmd.Flags().Lookup("device"))
+	viper.BindPFlag("download.ipsw.model", downloadIpswCmd.Flags().Lookup("model"))
+	viper.BindPFlag("download.ipsw.version", downloadIpswCmd.Flags().Lookup("version"))
+	viper.BindPFlag("download.ipsw.build", downloadIpswCmd.Flags().Lookup("build"))
+	// Bind IPSW-specific flags
+	viper.BindPFlag("download.ipsw.latest", downloadIpswCmd.Flags().Lookup("latest"))
+	viper.BindPFlag("download.ipsw.show-latest-version", downloadIpswCmd.Flags().Lookup("show-latest-version"))
+	viper.BindPFlag("download.ipsw.show-latest-build", downloadIpswCmd.Flags().Lookup("show-latest-build"))
+	viper.BindPFlag("download.ipsw.macos", downloadIpswCmd.Flags().Lookup("macos"))
+	viper.BindPFlag("download.ipsw.ibridge", downloadIpswCmd.Flags().Lookup("ibridge"))
+	viper.BindPFlag("download.ipsw.kernel", downloadIpswCmd.Flags().Lookup("kernel"))
+	viper.BindPFlag("download.ipsw.dyld", downloadIpswCmd.Flags().Lookup("dyld"))
+	viper.BindPFlag("download.ipsw.dyld-arch", downloadIpswCmd.Flags().Lookup("dyld-arch"))
+	// viper.BindPFlag("download.ipsw.kernel-spec", downloadIpswCmd.Flags().Lookup("kernel-spec"))
+	viper.BindPFlag("download.ipsw.pattern", downloadIpswCmd.Flags().Lookup("pattern"))
+	viper.BindPFlag("download.ipsw.fcs-keys", downloadIpswCmd.Flags().Lookup("fcs-keys"))
+	viper.BindPFlag("download.ipsw.fcs-keys-json", downloadIpswCmd.Flags().Lookup("fcs-keys-json"))
+	viper.BindPFlag("download.ipsw.decrypt", downloadIpswCmd.Flags().Lookup("decrypt"))
+	viper.BindPFlag("download.ipsw.output", downloadIpswCmd.Flags().Lookup("output"))
+	viper.BindPFlag("download.ipsw.flat", downloadIpswCmd.Flags().Lookup("flat"))
+	viper.BindPFlag("download.ipsw.urls", downloadIpswCmd.Flags().Lookup("urls"))
+	viper.BindPFlag("download.ipsw.usb", downloadIpswCmd.Flags().Lookup("usb"))
 }
 
-// ipswCmd represents the ipsw command
-var ipswCmd = &cobra.Command{
-	Use:           "ipsw",
-	Aliases:       []string{"i"},
-	Short:         "Download and parse IPSW(s) from the internets",
-	SilenceUsage:  true,
+// downloadIpswCmd represents the ipsw command
+var downloadIpswCmd = &cobra.Command{
+	Use:     "ipsw",
+	Aliases: []string{"i"},
+	Short:   "Download and parse IPSW(s) from ipsw.me",
+	Example: heredoc.Doc(`
+		# Download latest iOS IPSWs for iPhone15,2
+		❯ ipsw download ipsw --device iPhone15,2 --latest
+
+		# Download specific iOS build with kernelcache extraction
+		❯ ipsw download ipsw --device iPhone14,2 --build 20G75 --kernel
+
+		# Get URLs only without downloading
+		❯ ipsw download ipsw --device iPhone15,2 --version 17.0 --urls
+	`),
 	SilenceErrors: true,
 	RunE: func(cmd *cobra.Command, args []string) error {
 
@@ -107,35 +148,20 @@ var ipswCmd = &cobra.Command{
 		}
 		color.NoColor = viper.GetBool("no-color")
 
-		viper.BindPFlag("download.proxy", cmd.Flags().Lookup("proxy"))
-		viper.BindPFlag("download.insecure", cmd.Flags().Lookup("insecure"))
-		viper.BindPFlag("download.confirm", cmd.Flags().Lookup("confirm"))
-		viper.BindPFlag("download.skip-all", cmd.Flags().Lookup("skip-all"))
-		viper.BindPFlag("download.resume-all", cmd.Flags().Lookup("resume-all"))
-		viper.BindPFlag("download.restart-all", cmd.Flags().Lookup("restart-all"))
-		viper.BindPFlag("download.remove-commas", cmd.Flags().Lookup("remove-commas"))
-		viper.BindPFlag("download.white-list", cmd.Flags().Lookup("white-list"))
-		viper.BindPFlag("download.black-list", cmd.Flags().Lookup("black-list"))
-		viper.BindPFlag("download.device", cmd.Flags().Lookup("device"))
-		viper.BindPFlag("download.model", cmd.Flags().Lookup("model"))
-		viper.BindPFlag("download.version", cmd.Flags().Lookup("version"))
-		viper.BindPFlag("download.build", cmd.Flags().Lookup("build"))
-
 		// settings
-		proxy := viper.GetString("download.proxy")
-		insecure := viper.GetBool("download.insecure")
-		confirm := viper.GetBool("download.confirm")
-		skipAll := viper.GetBool("download.skip-all")
-		resumeAll := viper.GetBool("download.resume-all")
-		restartAll := viper.GetBool("download.restart-all")
-		removeCommas := viper.GetBool("download.remove-commas")
+		proxy := viper.GetString("download.ipsw.proxy")
+		insecure := viper.GetBool("download.ipsw.insecure")
+		confirm := viper.GetBool("download.ipsw.confirm")
+		skipAll := viper.GetBool("download.ipsw.skip-all")
+		resumeAll := viper.GetBool("download.ipsw.resume-all")
+		restartAll := viper.GetBool("download.ipsw.restart-all")
+		removeCommas := viper.GetBool("download.ipsw.remove-commas")
 		// filters
-		device := viper.GetString("download.device")
-		// model := viper.GetString("download.model")
-		// version := viper.GetString("download.version")
-		// build := viper.GetString("download.build")
-		doDownload := viper.GetStringSlice("download.white-list")
-		doNotDownload := viper.GetStringSlice("download.black-list")
+		device := viper.GetString("download.ipsw.device")
+		version := viper.GetString("download.ipsw.version")
+		build := viper.GetString("download.ipsw.build")
+		doDownload := viper.GetStringSlice("download.ipsw.white-list")
+		doNotDownload := viper.GetStringSlice("download.ipsw.black-list")
 		// flags
 		latest := viper.GetBool("download.ipsw.latest")
 		showLatestVersion := viper.GetBool("download.ipsw.show-latest-version")
@@ -152,8 +178,13 @@ var ipswCmd = &cobra.Command{
 		decrypt := viper.GetBool("download.ipsw.decrypt")
 		output := viper.GetString("download.ipsw.output")
 		flat := viper.GetBool("download.ipsw.flat")
-
 		// verify args
+		if len(device) == 0 && len(version) == 0 && len(build) == 0 {
+			return fmt.Errorf("you must also supply a --device || --version || --build (or use --latest)")
+		}
+		if len(version) > 0 && len(build) > 0 {
+			return fmt.Errorf("you cannot supply --version AND --build (they are mutually exclusive)")
+		}
 		if len(dyldArches) > 0 && !remoteDSC {
 			return errors.New("--dyld-arch can only be used with --dyld")
 		}
@@ -174,7 +205,7 @@ var ipswCmd = &cobra.Command{
 			}
 			device = dev.ProductType
 			if !latest {
-				dFlg.Build = dev.BuildVersion
+				build = dev.BuildVersion
 			}
 		}
 
@@ -291,7 +322,7 @@ var ipswCmd = &cobra.Command{
 			}
 
 			if len(filteredBuilds) == 0 {
-				log.Fatal(fmt.Sprintf("no IPSWs match device(s) %s %s", device, strings.Join(doDownload, " ")))
+				return fmt.Errorf(fmt.Sprintf("no IPSWs match device(s) %s %s", device, strings.Join(doDownload, " ")))
 			}
 
 			// convert from itunes to ipsw
@@ -306,11 +337,87 @@ var ipswCmd = &cobra.Command{
 				})
 			}
 		} else {
-			ipsws, err = filterIPSWs(cmd, macos)
-			if err != nil {
-				log.Fatal(err.Error())
+			// Filter IPSWs based on provided criteria
+			if len(version) > 0 {
+				ipsws, err = download.GetAllIPSW(version)
+				if err != nil {
+					return fmt.Errorf("failed to query ipsw.me api for ALL ipsws for version %s: %v", version, err)
+				}
+			} else if len(build) > 0 {
+				version, err = download.GetVersion(build)
+				if err != nil {
+					return fmt.Errorf("failed to query ipsw.me api for buildID %s => version: %v", build, err)
+				}
+				ipsws, err = download.GetAllIPSW(version)
+				if err != nil {
+					return fmt.Errorf("failed to query ipsw.me api for ALL ipsws for version %s: %v", version, err)
+				}
+				var buildFiltered []download.IPSW
+				for _, i := range ipsws {
+					if strings.EqualFold(build, i.BuildID) {
+						buildFiltered = append(buildFiltered, i)
+					}
+				}
+				ipsws = buildFiltered
+			} else if len(device) > 0 {
+				ipsws, err = download.GetDeviceIPSWs(device)
+				if err != nil {
+					return fmt.Errorf("failed to query ipsw.me api for device %s: %v", device, err)
+				}
 			}
-		}
+
+			var filteredIPSWs []download.IPSW
+			for _, i := range ipsws {
+				if len(device) > 0 {
+					if strings.EqualFold(device, i.Identifier) {
+						filteredIPSWs = append(filteredIPSWs, i)
+					}
+				} else {
+					if len(doDownload) > 0 {
+						for _, doDown := range doDownload {
+							if strings.HasPrefix(strings.ToLower(i.Identifier), strings.ToLower(doDown)) {
+								filteredIPSWs = append(filteredIPSWs, i)
+							}
+						}
+					} else if len(doNotDownload) > 0 {
+						for _, dontDown := range doNotDownload {
+							if !strings.HasPrefix(strings.ToLower(i.Identifier), strings.ToLower(dontDown)) {
+								filteredIPSWs = append(filteredIPSWs, i)
+							}
+						}
+					} else {
+						filteredIPSWs = append(filteredIPSWs, i)
+					}
+				}
+			}
+
+			if macos {
+				var furtherFilteredIPSWs []download.IPSW
+				for _, i := range filteredIPSWs {
+					if strings.Contains(i.Identifier, "Mac") {
+						furtherFilteredIPSWs = append(furtherFilteredIPSWs, i)
+					}
+				}
+				filteredIPSWs = furtherFilteredIPSWs
+			}
+
+			unique := make(map[string]bool, len(filteredIPSWs))
+			var uniqueIPSWs []download.IPSW
+			for _, i := range filteredIPSWs {
+				if len(i.URL) != 0 {
+					if !unique[i.URL] {
+						uniqueIPSWs = append(uniqueIPSWs, i)
+						unique[i.URL] = true
+					}
+				}
+			}
+
+			if len(uniqueIPSWs) == 0 {
+				return fmt.Errorf("filter flags matched 0 IPSWs")
+			}
+
+			ipsws = uniqueIPSWs
+		} // END IPSW FILTERING
 
 		if viper.GetBool("download.ipsw.urls") {
 			for _, i := range ipsws {
@@ -331,7 +438,13 @@ var ipswCmd = &cobra.Command{
 				prompt := &survey.Confirm{
 					Message: fmt.Sprintf("You are about to download %d IPSW files. Continue?", len(ipsws)),
 				}
-				survey.AskOne(prompt, &cont)
+				if err := survey.AskOne(prompt, &cont); err != nil {
+					if err == terminal.InterruptErr {
+						log.Warn("Exiting...")
+						return nil
+					}
+					return err
+				}
 			}
 		}
 
