@@ -30,6 +30,8 @@ import (
 	"strings"
 
 	"github.com/AlecAivazis/survey/v2"
+	"github.com/AlecAivazis/survey/v2/terminal"
+	"github.com/MakeNowJust/heredoc/v2"
 	"github.com/apex/log"
 	"github.com/blacktop/ipsw/internal/commands/extract"
 	"github.com/blacktop/ipsw/internal/download"
@@ -55,67 +57,103 @@ var otaDlCmdPlatforms = []string{
 }
 
 func init() {
-	DownloadCmd.AddCommand(otaDLCmd)
-
-	otaDLCmd.Flags().StringP("platform", "p", "", "Platform to download (ios, watchos, tvos, audioos || accessory, macos, recovery)")
-	otaDLCmd.RegisterFlagCompletionFunc("platform", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	DownloadCmd.AddCommand(downloadOtaCmd)
+	// Download behavior flags
+	downloadOtaCmd.Flags().String("proxy", "", "HTTP/HTTPS proxy")
+	downloadOtaCmd.Flags().Bool("insecure", false, "do not verify ssl certs")
+	downloadOtaCmd.Flags().BoolP("confirm", "y", false, "do not prompt user for confirmation")
+	downloadOtaCmd.Flags().Bool("skip-all", false, "always skip resumable IPSWs")
+	downloadOtaCmd.Flags().Bool("resume-all", false, "always resume resumable IPSWs")
+	downloadOtaCmd.Flags().Bool("restart-all", false, "always restart resumable IPSWs")
+	downloadOtaCmd.Flags().BoolP("remove-commas", "_", false, "replace commas in IPSW filename with underscores")
+	// Filter flags
+	downloadOtaCmd.Flags().StringArray("white-list", []string{}, "iOS device white list")
+	downloadOtaCmd.Flags().StringArray("black-list", []string{}, "iOS device black list")
+	downloadOtaCmd.Flags().StringP("device", "d", "", "iOS Device (i.e. iPhone11,2)")
+	downloadOtaCmd.Flags().StringP("model", "m", "", "iOS Model (i.e. D321AP)")
+	downloadOtaCmd.Flags().StringP("version", "v", "", "iOS Version (i.e. 12.3.1)")
+	downloadOtaCmd.Flags().StringP("build", "b", "", "iOS BuildID (i.e. 16F203)")
+	// OTA-specific flags
+	downloadOtaCmd.Flags().String("platform", "", "Platform to download (ios, watchos, tvos, audioos || accessory, macos, recovery)")
+	downloadOtaCmd.RegisterFlagCompletionFunc("platform", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return otaDlCmdPlatforms, cobra.ShellCompDirectiveDefault
 	})
-	otaDLCmd.Flags().Bool("beta", false, "Download Beta OTAs")
-	otaDLCmd.Flags().Bool("latest", false, "Download latest OTAs")
-	otaDLCmd.Flags().Bool("delta", false, "Download Delta OTAs")
-	otaDLCmd.Flags().Bool("rsr", false, "Download Rapid Security Response OTAs")
-	otaDLCmd.Flags().Bool("sim", false, "Download Simulator OTAs")
-	otaDLCmd.Flags().BoolP("kernel", "k", false, "Extract kernelcache from remote OTA zip")
-	otaDLCmd.Flags().Bool("dyld", false, "Extract dyld_shared_cache(s) from remote OTA zip")
-	otaDLCmd.Flags().BoolP("urls", "u", false, "Dump URLs only")
-	otaDLCmd.Flags().BoolP("json", "j", false, "Dump URLs as JSON only")
-	otaDLCmd.Flags().StringArrayP("dyld-arch", "a", []string{}, "dyld_shared_cache architecture(s) to remote extract")
-	otaDLCmd.RegisterFlagCompletionFunc("dyld-arch", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	downloadOtaCmd.Flags().Bool("beta", false, "Download Beta OTAs")
+	downloadOtaCmd.Flags().Bool("latest", false, "Download latest OTAs")
+	downloadOtaCmd.Flags().Bool("delta", false, "Download Delta OTAs")
+	downloadOtaCmd.Flags().Bool("rsr", false, "Download Rapid Security Response OTAs")
+	downloadOtaCmd.Flags().Bool("sim", false, "Download Simulator OTAs")
+	downloadOtaCmd.Flags().BoolP("kernel", "k", false, "Extract kernelcache from remote OTA zip")
+	downloadOtaCmd.Flags().Bool("dyld", false, "Extract dyld_shared_cache(s) from remote OTA zip")
+	downloadOtaCmd.Flags().BoolP("urls", "u", false, "Dump URLs only")
+	downloadOtaCmd.Flags().BoolP("json", "j", false, "Dump URLs as JSON only")
+	downloadOtaCmd.Flags().StringArrayP("dyld-arch", "a", []string{}, "dyld_shared_cache architecture(s) to remote extract")
+	downloadOtaCmd.RegisterFlagCompletionFunc("dyld-arch", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return dyld.DscArches, cobra.ShellCompDirectiveDefault
 	})
-	otaDLCmd.Flags().Bool("driver-kit", false, "Extract DriverKit dyld_shared_cache(s) from remote OTA zip")
-	otaDLCmd.Flags().String("pattern", "", "Download remote files that match regex")
-	otaDLCmd.Flags().BoolP("flat", "f", false, "Do NOT perserve directory structure when downloading with --pattern")
-	otaDLCmd.Flags().Bool("info", false, "Show all the latest OTAs available")
-	otaDLCmd.Flags().StringP("output", "o", "", "Folder to download files to")
-	otaDLCmd.MarkFlagDirname("output")
-	otaDLCmd.Flags().Bool("show-latest-version", false, "Show latest iOS version")
-	otaDLCmd.Flags().Bool("show-latest-build", false, "Show latest iOS build")
-	otaDLCmd.MarkFlagsMutuallyExclusive("info", "beta", "latest")
-	viper.BindPFlag("download.ota.platform", otaDLCmd.Flags().Lookup("platform"))
-	viper.BindPFlag("download.ota.beta", otaDLCmd.Flags().Lookup("beta"))
-	viper.BindPFlag("download.ota.latest", otaDLCmd.Flags().Lookup("latest"))
-	viper.BindPFlag("download.ota.delta", otaDLCmd.Flags().Lookup("delta"))
-	viper.BindPFlag("download.ota.rsr", otaDLCmd.Flags().Lookup("rsr"))
-	viper.BindPFlag("download.ota.sim", otaDLCmd.Flags().Lookup("sim"))
-	viper.BindPFlag("download.ota.dyld", otaDLCmd.Flags().Lookup("dyld"))
-	viper.BindPFlag("download.ota.urls", otaDLCmd.Flags().Lookup("urls"))
-	viper.BindPFlag("download.ota.json", otaDLCmd.Flags().Lookup("json"))
-	viper.BindPFlag("download.ota.dyld-arch", otaDLCmd.Flags().Lookup("dyld-arch"))
-	viper.BindPFlag("download.ota.driver-kit", otaDLCmd.Flags().Lookup("driver-kit"))
-	viper.BindPFlag("download.ota.kernel", otaDLCmd.Flags().Lookup("kernel"))
-	viper.BindPFlag("download.ota.pattern", otaDLCmd.Flags().Lookup("pattern"))
-	viper.BindPFlag("download.ota.flat", otaDLCmd.Flags().Lookup("flat"))
-	viper.BindPFlag("download.ota.info", otaDLCmd.Flags().Lookup("info"))
-	viper.BindPFlag("download.ota.output", otaDLCmd.Flags().Lookup("output"))
-	viper.BindPFlag("download.ota.show-latest-version", otaDLCmd.Flags().Lookup("show-latest-version"))
-	viper.BindPFlag("download.ota.show-latest-build", otaDLCmd.Flags().Lookup("show-latest-build"))
+	downloadOtaCmd.Flags().Bool("driver-kit", false, "Extract DriverKit dyld_shared_cache(s) from remote OTA zip")
+	downloadOtaCmd.Flags().String("pattern", "", "Download remote files that match regex")
+	downloadOtaCmd.Flags().BoolP("flat", "f", false, "Do NOT perserve directory structure when downloading with --pattern")
+	downloadOtaCmd.Flags().Bool("info", false, "Show all the latest OTAs available")
+	downloadOtaCmd.Flags().StringP("output", "o", "", "Folder to download files to")
+	downloadOtaCmd.MarkFlagDirname("output")
+	downloadOtaCmd.Flags().Bool("show-latest-version", false, "Show latest iOS version")
+	downloadOtaCmd.Flags().Bool("show-latest-build", false, "Show latest iOS build")
+	downloadOtaCmd.MarkFlagsMutuallyExclusive("info", "beta", "latest")
+	// Bind download behavior flags
+	viper.BindPFlag("download.ota.proxy", downloadOtaCmd.Flags().Lookup("proxy"))
+	viper.BindPFlag("download.ota.insecure", downloadOtaCmd.Flags().Lookup("insecure"))
+	viper.BindPFlag("download.ota.confirm", downloadOtaCmd.Flags().Lookup("confirm"))
+	viper.BindPFlag("download.ota.skip-all", downloadOtaCmd.Flags().Lookup("skip-all"))
+	viper.BindPFlag("download.ota.resume-all", downloadOtaCmd.Flags().Lookup("resume-all"))
+	viper.BindPFlag("download.ota.restart-all", downloadOtaCmd.Flags().Lookup("restart-all"))
+	viper.BindPFlag("download.ota.remove-commas", downloadOtaCmd.Flags().Lookup("remove-commas"))
+	// Bind filter flags
+	viper.BindPFlag("download.ota.white-list", downloadOtaCmd.Flags().Lookup("white-list"))
+	viper.BindPFlag("download.ota.black-list", downloadOtaCmd.Flags().Lookup("black-list"))
+	viper.BindPFlag("download.ota.device", downloadOtaCmd.Flags().Lookup("device"))
+	viper.BindPFlag("download.ota.model", downloadOtaCmd.Flags().Lookup("model"))
+	viper.BindPFlag("download.ota.version", downloadOtaCmd.Flags().Lookup("version"))
+	viper.BindPFlag("download.ota.build", downloadOtaCmd.Flags().Lookup("build"))
+	// Bind OTA-specific flags
+	viper.BindPFlag("download.ota.platform", downloadOtaCmd.Flags().Lookup("platform"))
+	viper.BindPFlag("download.ota.beta", downloadOtaCmd.Flags().Lookup("beta"))
+	viper.BindPFlag("download.ota.latest", downloadOtaCmd.Flags().Lookup("latest"))
+	viper.BindPFlag("download.ota.delta", downloadOtaCmd.Flags().Lookup("delta"))
+	viper.BindPFlag("download.ota.rsr", downloadOtaCmd.Flags().Lookup("rsr"))
+	viper.BindPFlag("download.ota.sim", downloadOtaCmd.Flags().Lookup("sim"))
+	viper.BindPFlag("download.ota.dyld", downloadOtaCmd.Flags().Lookup("dyld"))
+	viper.BindPFlag("download.ota.urls", downloadOtaCmd.Flags().Lookup("urls"))
+	viper.BindPFlag("download.ota.json", downloadOtaCmd.Flags().Lookup("json"))
+	viper.BindPFlag("download.ota.dyld-arch", downloadOtaCmd.Flags().Lookup("dyld-arch"))
+	viper.BindPFlag("download.ota.driver-kit", downloadOtaCmd.Flags().Lookup("driver-kit"))
+	viper.BindPFlag("download.ota.kernel", downloadOtaCmd.Flags().Lookup("kernel"))
+	viper.BindPFlag("download.ota.pattern", downloadOtaCmd.Flags().Lookup("pattern"))
+	viper.BindPFlag("download.ota.flat", downloadOtaCmd.Flags().Lookup("flat"))
+	viper.BindPFlag("download.ota.info", downloadOtaCmd.Flags().Lookup("info"))
+	viper.BindPFlag("download.ota.output", downloadOtaCmd.Flags().Lookup("output"))
+	viper.BindPFlag("download.ota.show-latest-version", downloadOtaCmd.Flags().Lookup("show-latest-version"))
+	viper.BindPFlag("download.ota.show-latest-build", downloadOtaCmd.Flags().Lookup("show-latest-build"))
 }
 
-// otaDLCmd represents the ota download command
-var otaDLCmd = &cobra.Command{
+// downloadOtaCmd represents the ota download command
+var downloadOtaCmd = &cobra.Command{
 	Use:     "ota [options]",
 	Aliases: []string{"o"},
 	Short:   "Download OTAs",
-	Example: `  # Download the iOS 14.8.1 OTA for the iPhone10,1
-  ❯ ipsw download ota --platform ios --version 14.8.1 --device iPhone10,1
-    ? You are about to download 1 OTA files. Continue? Yes
-	  • Getting OTA               build=18H107 device=iPhone10,1 version=iOS1481Short
-	  280.0 MiB / 3.7 GiB [===>------------------------------------------------------| 51m18s
-  # Get all the latest BETA iOS OTAs URLs as JSON
-  ❯ ipsw download ota --platform ios --beta --urls --json`,
-	SilenceUsage:  true,
+	Example: heredoc.Doc(`
+		# Download the iOS 14.8.1 OTA for the iPhone10,1
+		❯ ipsw download ota --platform ios --version 14.8.1 --device iPhone10,1
+
+		# Get all the latest BETA iOS OTAs URLs as JSON
+		❯ ipsw download ota --platform ios --beta --urls --json
+
+		# Download latest tvOS OTA and extract kernelcache
+		❯ ipsw download ota --platform tvos --latest --kernel
+
+		# Download Xcode Simulator Runtime OTAs
+		❯ ipsw download ota --platform ios --sim --build "22F77"
+	`),
 	SilenceErrors: true,
 	RunE: func(cmd *cobra.Command, args []string) error {
 
@@ -127,35 +165,21 @@ var otaDLCmd = &cobra.Command{
 		}
 		color.NoColor = viper.GetBool("no-color")
 
-		viper.BindPFlag("download.proxy", cmd.Flags().Lookup("proxy"))
-		viper.BindPFlag("download.insecure", cmd.Flags().Lookup("insecure"))
-		viper.BindPFlag("download.confirm", cmd.Flags().Lookup("confirm"))
-		viper.BindPFlag("download.skip-all", cmd.Flags().Lookup("skip-all"))
-		viper.BindPFlag("download.resume-all", cmd.Flags().Lookup("resume-all"))
-		viper.BindPFlag("download.restart-all", cmd.Flags().Lookup("restart-all"))
-		viper.BindPFlag("download.remove-commas", cmd.Flags().Lookup("remove-commas"))
-		viper.BindPFlag("download.white-list", cmd.Flags().Lookup("white-list"))
-		viper.BindPFlag("download.black-list", cmd.Flags().Lookup("black-list"))
-		viper.BindPFlag("download.device", cmd.Flags().Lookup("device"))
-		viper.BindPFlag("download.model", cmd.Flags().Lookup("model"))
-		viper.BindPFlag("download.version", cmd.Flags().Lookup("version"))
-		viper.BindPFlag("download.build", cmd.Flags().Lookup("build"))
-
 		// settings
-		proxy := viper.GetString("download.proxy")
-		insecure := viper.GetBool("download.insecure")
-		confirm := viper.GetBool("download.confirm")
-		skipAll := viper.GetBool("download.skip-all")
-		resumeAll := viper.GetBool("download.resume-all")
-		restartAll := viper.GetBool("download.restart-all")
-		removeCommas := viper.GetBool("download.remove-commas")
+		proxy := viper.GetString("download.ota.proxy")
+		insecure := viper.GetBool("download.ota.insecure")
+		confirm := viper.GetBool("download.ota.confirm")
+		skipAll := viper.GetBool("download.ota.skip-all")
+		resumeAll := viper.GetBool("download.ota.resume-all")
+		restartAll := viper.GetBool("download.ota.restart-all")
+		removeCommas := viper.GetBool("download.ota.remove-commas")
 		// filters
-		device := viper.GetString("download.device")
-		model := viper.GetString("download.model")
-		version := viper.GetString("download.version")
-		build := viper.GetString("download.build")
-		doDownload := viper.GetStringSlice("download.white-list")
-		doNotDownload := viper.GetStringSlice("download.black-list")
+		device := viper.GetString("download.ota.device")
+		model := viper.GetString("download.ota.model")
+		version := viper.GetString("download.ota.version")
+		build := viper.GetString("download.ota.build")
+		doDownload := viper.GetStringSlice("download.ota.white-list")
+		doNotDownload := viper.GetStringSlice("download.ota.black-list")
 		// flags
 		platform := viper.GetString("download.ota.platform")
 		getBeta := viper.GetBool("download.ota.beta")
@@ -211,7 +235,7 @@ var otaDLCmd = &cobra.Command{
 		// Query for asset sets
 		as, err := download.GetAssetSets(proxy, insecure)
 		if err != nil {
-			log.Fatal(err.Error())
+			return err
 		}
 
 		/****************
@@ -278,7 +302,7 @@ var otaDLCmd = &cobra.Command{
 		if len(version) > 0 {
 			ver, err = semver.NewVersion(version)
 			if err != nil {
-				log.Fatal("failed to convert version into semver object")
+				return fmt.Errorf("failed to convert version into semver object")
 			}
 		}
 
@@ -305,7 +329,7 @@ var otaDLCmd = &cobra.Command{
 
 		otas, err := otaXML.GetPallasOTAs()
 		if err != nil {
-			return err
+			return fmt.Errorf("failed to get Pallas OTAs: %v", err)
 		}
 
 		if showLatestVersion {
@@ -364,7 +388,10 @@ var otaDLCmd = &cobra.Command{
 				prompt := &survey.Confirm{
 					Message: fmt.Sprintf("You are about to download %d OTA files. Continue?", len(otas)),
 				}
-				survey.AskOne(prompt, &cont)
+				if err := survey.AskOne(prompt, &cont); err == terminal.InterruptErr {
+					log.Warn("Exiting...")
+					return nil
+				}
 			}
 		}
 
@@ -445,7 +472,9 @@ var otaDLCmd = &cobra.Command{
 					if getSim {
 						folder = filepath.Join(destPath, fmt.Sprintf("%s_%s_Simulator_OTAs", strings.ToUpper(platform), o.SimulatorVersion))
 					}
-					os.MkdirAll(folder, 0750)
+					if err := os.MkdirAll(folder, 0750); err != nil {
+						return fmt.Errorf("failed to create folder %s: %v", folder, err)
+					}
 					var devices string
 					if len(o.SupportedDevices) > 0 {
 						sort.Strings(o.SupportedDevices)
