@@ -28,6 +28,7 @@ import (
 	"text/tabwriter"
 	"time"
 
+	"github.com/MakeNowJust/heredoc/v2"
 	"github.com/apex/log"
 	"github.com/blacktop/ipsw/internal/download"
 	"github.com/blacktop/ipsw/internal/utils"
@@ -38,34 +39,31 @@ import (
 )
 
 func init() {
-	DownloadCmd.AddCommand(rssCmd)
+	DownloadCmd.AddCommand(downloadRssCmd)
 
-	rssCmd.Flags().BoolP("watch", "w", false, "Watch for NEW releases")
-	rssCmd.Flags().BoolP("json", "j", false, "Output as JSON")
-	rssCmd.SetHelpFunc(func(c *cobra.Command, s []string) {
-		DownloadCmd.PersistentFlags().MarkHidden("white-list")
-		DownloadCmd.PersistentFlags().MarkHidden("black-list")
-		DownloadCmd.PersistentFlags().MarkHidden("device")
-		DownloadCmd.PersistentFlags().MarkHidden("model")
-		DownloadCmd.PersistentFlags().MarkHidden("version")
-		DownloadCmd.PersistentFlags().MarkHidden("build")
-		DownloadCmd.PersistentFlags().MarkHidden("confirm")
-		DownloadCmd.PersistentFlags().MarkHidden("skip-all")
-		DownloadCmd.PersistentFlags().MarkHidden("resume-all")
-		DownloadCmd.PersistentFlags().MarkHidden("restart-all")
-		DownloadCmd.PersistentFlags().MarkHidden("remove-commas")
-		c.Parent().HelpFunc()(c, s)
-	})
-	viper.BindPFlag("download.rss.watch", rssCmd.Flags().Lookup("watch"))
-	viper.BindPFlag("download.rss.json", rssCmd.Flags().Lookup("json"))
+	downloadRssCmd.Flags().BoolP("watch", "w", false, "Watch for NEW releases")
+	downloadRssCmd.Flags().BoolP("json", "j", false, "Output as JSON")
+	viper.BindPFlag("download.rss.watch", downloadRssCmd.Flags().Lookup("watch"))
+	viper.BindPFlag("download.rss.json", downloadRssCmd.Flags().Lookup("json"))
 }
 
-// rssCmd represents the rss command
-var rssCmd = &cobra.Command{
+// downloadRssCmd represents the rss command
+var downloadRssCmd = &cobra.Command{
 	Use:     "rss",
 	Aliases: []string{"r"},
 	Short:   "Read Releases - Apple Developer RSS Feed",
-	Run: func(cmd *cobra.Command, args []string) {
+	Example: heredoc.Doc(`
+		# Read latest Apple developer releases
+		❯ ipsw download rss
+
+		# Watch for new releases with notifications
+		❯ ipsw download rss --watch
+
+		# Output RSS feed as JSON
+		❯ ipsw download rss --json
+	`),
+	SilenceErrors: true,
+	RunE: func(cmd *cobra.Command, args []string) error {
 		var releases []string
 
 		if viper.GetBool("verbose") {
@@ -79,7 +77,7 @@ var rssCmd = &cobra.Command{
 
 		rss, err := download.GetRSS()
 		if err != nil {
-			log.Fatal(err.Error())
+			return err
 		}
 		for _, item := range rss.Channel.Items {
 			releases = append(releases, fmt.Sprintf("%s - %s", item.Title, item.PubDate))
@@ -93,7 +91,7 @@ var rssCmd = &cobra.Command{
 				// check for NEW releases
 				rss, err := download.GetRSS()
 				if err != nil {
-					log.Fatal(err.Error())
+					return err
 				}
 
 				for _, item := range rss.Channel.Items {
@@ -102,7 +100,7 @@ var rssCmd = &cobra.Command{
 						releases = append(releases, fmt.Sprintf("%s - %s", item.Title, item.PubDate))
 
 						if err := beeep.Alert("🆕 Apple - Release", fmt.Sprintf("%s - %s", item.Title, item.PubDate), "assets/warning.png"); err != nil {
-							log.Fatal(err.Error())
+							return err
 						}
 					}
 				}
@@ -112,7 +110,7 @@ var rssCmd = &cobra.Command{
 		if asJSON {
 			rssJSON, err := json.Marshal(rss)
 			if err != nil {
-				log.Fatal(err.Error())
+				return err
 			}
 			fmt.Println(string(rssJSON))
 		} else {
@@ -123,12 +121,14 @@ var rssCmd = &cobra.Command{
 			for _, item := range rss.Channel.Items {
 				date, err := item.PubDate.GetDate()
 				if err != nil {
-					log.Fatal(err.Error())
+					return err
 				}
 				fmt.Fprintf(w, "- %s\t<%s>\t%s  \n", item.Title, date.Format("Mon, 02Jan2006 15:04:05 MST"), item.Link)
 			}
 			w.Flush()
 			fmt.Println()
 		}
+
+		return nil
 	},
 }
