@@ -1,7 +1,7 @@
 //go:build darwin
 
 /*
-Copyright © 2024 blacktop
+Copyright © 2025 blacktop
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -114,7 +114,7 @@ var iaCmd = &cobra.Command{
 			}).Info, 2)("OTA")
 		}
 
-		utils.Indent(log.Info, 2)(fmt.Sprintf("Run:\n\tipsw ota ls %s | grep \"dyld\\|cryptex-system-arm64e\"", zips[0]))
+		utils.Indent(log.Info, 2)(fmt.Sprintf("Run:\n\tipsw ota ls %s | grep \"dyld\\|cryptex-system\"", zips[0]))
 
 		// block until user hits ctrl-c
 		done := make(chan os.Signal, 1)
