@@ -41,11 +41,10 @@ func init() {
 // lsbomCmd represents the lsbom command
 var lsbomCmd = &cobra.Command{
 	Use:           "lsbom",
-	Short:         "List contents of a bom file",
+	Short:         "List contents of a BOM file",
 	Args:          cobra.ExactArgs(1),
 	SilenceUsage:  true,
 	SilenceErrors: true,
-	Hidden:        true,
 	RunE: func(cmd *cobra.Command, args []string) error {
 
 		if Verbose {
