@@ -29,13 +29,13 @@ import (
 	"strings"
 	"time"
 
-	"github.com/AlecAivazis/survey/v2"
 	"github.com/alecthomas/chroma/v2/styles"
 	"github.com/apex/log"
 	"github.com/blacktop/go-macho"
 	"github.com/blacktop/go-macho/types"
 	"github.com/blacktop/ipsw/internal/ai"
 	dcmd "github.com/blacktop/ipsw/internal/commands/disass"
+	mcmd "github.com/blacktop/ipsw/internal/commands/macho"
 	"github.com/blacktop/ipsw/internal/magic"
 	"github.com/blacktop/ipsw/pkg/disass"
 	"github.com/caarlos0/ctrlc"
@@ -119,6 +119,7 @@ var machoDisassCmd = &cobra.Command{
 	SilenceErrors: true,
 	RunE: func(cmd *cobra.Command, args []string) error {
 
+		var err error
 		var m *macho.File
 		var ms []*macho.File
 		var middleAddr uint64
@@ -171,50 +172,12 @@ var machoDisassCmd = &cobra.Command{
 			return fmt.Errorf("failed to detect file type: %v", err)
 		}
 
-		fat, err := macho.OpenFat(machoPath)
-		if err != nil && err != macho.ErrNotFat {
-			log.Fatal(err.Error())
-		}
-		if err == macho.ErrNotFat {
-			m, err = macho.Open(machoPath)
-			if err != nil {
-				log.Fatal(err.Error())
-			}
-		} else {
-			var options []string
-			var shortOptions []string
-			for _, arch := range fat.Arches {
-				if strings.Contains(strings.ToLower(arch.SubCPU.String(arch.CPU)), "arm64") {
-					options = append(options, fmt.Sprintf("%s, %s", arch.CPU, arch.SubCPU.String(arch.CPU)))
-					shortOptions = append(shortOptions, strings.ToLower(arch.SubCPU.String(arch.CPU)))
-					m = arch.File //
-				}
-			}
-
-			if len(shortOptions) > 1 {
-				if len(selectedArch) > 0 {
-					found := false
-					for i, opt := range shortOptions {
-						if strings.Contains(strings.ToLower(opt), strings.ToLower(selectedArch)) {
-							m = fat.Arches[i].File
-							found = true
-							break
-						}
-					}
-					if !found {
-						return fmt.Errorf("--arch '%s' not found in: %s", selectedArch, strings.Join(shortOptions, ", "))
-					}
-				} else {
-					choice := 0
-					prompt := &survey.Select{
-						Message: "Detected a universal MachO file, please select an architecture to analyze:",
-						Options: options,
-					}
-					survey.AskOne(prompt, &choice)
-					m = fat.Arches[choice].File
-				}
-			}
+		mr, err := mcmd.OpenMachO(machoPath, selectedArch)
+		if err != nil {
+			return err
 		}
+		defer mr.Close()
+		m = mr.File
 
 		if !strings.Contains(strings.ToLower(m.FileHeader.SubCPU.String(m.CPU)), "arm64") {
 			return fmt.Errorf("can only disassemble arm64 binaries")
