@@ -27,7 +27,6 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/AlecAivazis/survey/v2"
 	"github.com/alecthomas/chroma/v2/styles"
 	"github.com/apex/log"
 	"github.com/blacktop/go-macho"
@@ -130,7 +129,7 @@ var swiftDumpCmd = &cobra.Command{
 		}
 
 		doDemangle := viper.GetBool("swift-dump.demangle")
-		
+
 		// Auto-enable demangle and interface when headers is specified
 		if viper.GetBool("swift-dump.headers") {
 			doDemangle = true
@@ -153,46 +152,12 @@ var swiftDumpCmd = &cobra.Command{
 
 		if ok, _ := magic.IsMachO(args[0]); ok { /* MachO binary */
 			machoPath := filepath.Clean(args[0])
-			// first check for fat file
-			fat, err := macho.OpenFat(machoPath)
-			if err != nil && err != macho.ErrNotFat {
+			mr, err := mcmd.OpenMachO(machoPath, viper.GetString("swift-dump.arch"))
+			if err != nil {
 				return err
 			}
-			if err == macho.ErrNotFat {
-				m, err = macho.Open(machoPath)
-				if err != nil {
-					return err
-				}
-			} else {
-				var options []string
-				var shortOptions []string
-				for _, arch := range fat.Arches {
-					options = append(options, fmt.Sprintf("%s, %s", arch.CPU, arch.SubCPU.String(arch.CPU)))
-					shortOptions = append(shortOptions, strings.ToLower(arch.SubCPU.String(arch.CPU)))
-				}
-
-				if len(viper.GetString("swift-dump.arch")) > 0 {
-					found := false
-					for i, opt := range shortOptions {
-						if strings.Contains(strings.ToLower(opt), strings.ToLower(viper.GetString("swift-dump.arch"))) {
-							m = fat.Arches[i].File
-							found = true
-							break
-						}
-					}
-					if !found {
-						return fmt.Errorf("--arch '%s' not found in: %s", viper.GetString("swift-dump.arch"), strings.Join(shortOptions, ", "))
-					}
-				} else {
-					choice := 0
-					prompt := &survey.Select{
-						Message: "Detected a universal MachO file, please select an architecture to analyze:",
-						Options: options,
-					}
-					survey.AskOne(prompt, &choice)
-					m = fat.Arches[choice].File
-				}
-			}
+			defer mr.Close()
+			m = mr.File
 			if viper.GetBool("swift-dump.deps") {
 				log.Error("cannot dump imported private frameworks from a MachO file (only from a DSC)")
 			}
