@@ -9,6 +9,7 @@ import (
 	"fmt"
 	"io"
 	"net/http"
+	"os"
 	"path/filepath"
 	"regexp"
 	"strconv"
@@ -44,6 +45,9 @@ const (
 	ipadMini          = "iPad mini"
 	iphone            = "iPhone"
 	ipodTouch         = "iPod touch"
+
+	// Cache settings
+	cacheFileName = "firmware_keys_cache.json"
 )
 
 type Queue struct {
@@ -191,6 +195,188 @@ type WikiFWKey struct {
 	Kbag     []string `json:"kbag,omitempty"`
 }
 
+// CachedFWKey represents a simplified firmware key structure for caching
+type CachedFWKey struct {
+	Filename string `json:"filename,omitempty"`
+	Device   string `json:"device,omitempty"`
+	Key      string `json:"key,omitempty"`
+	Devkbag  string `json:"devkbag,omitempty"`
+	Iv       string `json:"iv,omitempty"`
+	Kbag     string `json:"kbag,omitempty"`
+}
+
+// CachedFWKeys represents cached firmware keys
+type CachedFWKeys map[string]CachedFWKey
+
+// FirmwareKeysCache represents the cache file structure
+type FirmwareKeysCache struct {
+	CachedAt time.Time                     `json:"cached_at"`
+	Entries  map[string]FirmwareKeysCacheEntry `json:"entries"`
+}
+
+// FirmwareKeysCacheEntry represents a cache entry for specific device/build combination
+type FirmwareKeysCacheEntry struct {
+	CachedAt time.Time    `json:"cached_at"`
+	Keys     CachedFWKeys `json:"keys"`
+}
+
+// GetKeyByFilename returns a key for a given filename from cached keys
+func (ks CachedFWKeys) GetKeyByFilename(filename string) (string, error) {
+	for _, wk := range ks {
+		if strings.EqualFold(wk.Filename, filepath.Base(filename)) {
+			if len(wk.Iv) > 0 && len(wk.Key) > 0 {
+				return fmt.Sprintf("%s%s", wk.Iv, wk.Key), nil
+			} else if len(wk.Key) > 0 {
+				return wk.Key, nil
+			}
+		}
+	}
+	return "", fmt.Errorf("no key found for file '%s'", filepath.Base(filename))
+}
+
+// GetKeyByRegex returns a key for a given regex pattern from cached keys
+func (ks CachedFWKeys) GetKeyByRegex(pattern string) (string, error) {
+	for _, wk := range ks {
+		if matched, _ := regexp.MatchString(pattern, wk.Filename); matched {
+			if len(wk.Iv) > 0 && len(wk.Key) > 0 {
+				return fmt.Sprintf("%s%s", wk.Iv, wk.Key), nil
+			} else if len(wk.Key) > 0 {
+				return wk.Key, nil
+			}
+		}
+	}
+	return "", fmt.Errorf("no key found for pattern '%s'", pattern)
+}
+
+// getCacheDir returns the cache directory path
+func getCacheDir() (string, error) {
+	configDir, err := os.UserConfigDir()
+	if err != nil {
+		return "", fmt.Errorf("failed to get user config directory: %w", err)
+	}
+	cacheDir := filepath.Join(configDir, "ipsw")
+	if err := os.MkdirAll(cacheDir, 0755); err != nil {
+		return "", fmt.Errorf("failed to create cache directory: %w", err)
+	}
+	return cacheDir, nil
+}
+
+// getCacheFilePath returns the full path to the cache file
+func getCacheFilePath() (string, error) {
+	cacheDir, err := getCacheDir()
+	if err != nil {
+		return "", err
+	}
+	return filepath.Join(cacheDir, cacheFileName), nil
+}
+
+// loadCache loads the firmware keys cache from disk
+func loadCache() (*FirmwareKeysCache, error) {
+	cachePath, err := getCacheFilePath()
+	if err != nil {
+		return nil, err
+	}
+
+	if _, err := os.Stat(cachePath); os.IsNotExist(err) {
+		// Cache file doesn't exist, return empty cache
+		return &FirmwareKeysCache{
+			CachedAt: time.Now(),
+			Entries:  make(map[string]FirmwareKeysCacheEntry),
+		}, nil
+	}
+
+	data, err := os.ReadFile(cachePath)
+	if err != nil {
+		return nil, fmt.Errorf("failed to read cache file: %w", err)
+	}
+
+	var cache FirmwareKeysCache
+	if err := json.Unmarshal(data, &cache); err != nil {
+		// If cache is corrupted, return empty cache
+		log.Warnf("cache file corrupted, starting fresh: %v", err)
+		return &FirmwareKeysCache{
+			CachedAt: time.Now(),
+			Entries:  make(map[string]FirmwareKeysCacheEntry),
+		}, nil
+	}
+
+	return &cache, nil
+}
+
+// saveCache saves the firmware keys cache to disk
+func saveCache(cache *FirmwareKeysCache) error {
+	cachePath, err := getCacheFilePath()
+	if err != nil {
+		return err
+	}
+
+	cache.CachedAt = time.Now()
+	data, err := json.MarshalIndent(cache, "", "  ")
+	if err != nil {
+		return fmt.Errorf("failed to marshal cache: %w", err)
+	}
+
+	if err := os.WriteFile(cachePath, data, 0644); err != nil {
+		return fmt.Errorf("failed to write cache file: %w", err)
+	}
+
+	return nil
+}
+
+// getCacheKey generates a unique cache key for device/build combination
+func getCacheKey(device, build string) string {
+	return fmt.Sprintf("%s_%s", device, build)
+}
+
+
+// convertWikiKeysToCache converts WikiFWKeys to CachedFWKeys
+func convertWikiKeysToCache(wikiKeys WikiFWKeys) CachedFWKeys {
+	cached := make(CachedFWKeys)
+	for key, wk := range wikiKeys {
+		cached[key] = CachedFWKey{
+			Filename: getFirstOrEmpty(wk.Filename),
+			Device:   getFirstOrEmpty(wk.Device),
+			Key:      getFirstOrEmpty(wk.Key),
+			Devkbag:  getFirstOrEmpty(wk.Devkbag),
+			Iv:       getFirstOrEmpty(wk.Iv),
+			Kbag:     getFirstOrEmpty(wk.Kbag),
+		}
+	}
+	return cached
+}
+
+// convertCacheToWikiKeys converts CachedFWKeys to WikiFWKeys for compatibility
+func convertCacheToWikiKeys(cached CachedFWKeys) WikiFWKeys {
+	wiki := make(WikiFWKeys)
+	for key, ck := range cached {
+		wiki[key] = WikiFWKey{
+			Filename: stringToSlice(ck.Filename),
+			Device:   stringToSlice(ck.Device),
+			Key:      stringToSlice(ck.Key),
+			Devkbag:  stringToSlice(ck.Devkbag),
+			Iv:       stringToSlice(ck.Iv),
+			Kbag:     stringToSlice(ck.Kbag),
+		}
+	}
+	return wiki
+}
+
+// getFirstOrEmpty returns the first element of a slice or empty string if slice is empty
+func getFirstOrEmpty(slice []string) string {
+	if len(slice) > 0 {
+		return slice[0]
+	}
+	return ""
+}
+
+// stringToSlice converts a string to a slice with that string as the only element (if not empty)
+func stringToSlice(s string) []string {
+	if s == "" {
+		return []string{}
+	}
+	return []string{s}
+}
+
 func (k WikiFWKey) String() string {
 	var out string
 	for i, fn := range k.Filename {
@@ -1363,6 +1549,47 @@ func GetWikiOTAs(cfg *WikiConfig, proxy string, insecure bool) ([]WikiFirmware,
 }
 
 func GetWikiFirmwareKeys(cfg *WikiConfig, proxy string, insecure bool) (WikiFWKeys, error) {
+	// Check cache first
+	cacheKey := getCacheKey(cfg.Device, cfg.Build)
+	cache, err := loadCache()
+	if err != nil {
+		log.Warnf("failed to load cache: %v", err)
+	} else {
+		if entry, exists := cache.Entries[cacheKey]; exists {
+			log.Debugf("found cached keys for %s %s", cfg.Device, cfg.Build)
+			return convertCacheToWikiKeys(entry.Keys), nil
+		}
+	}
+
+	log.Debugf("no cached keys found for %s %s, fetching from wiki", cfg.Device, cfg.Build)
+
+	// Fetch from wiki if not in cache
+	keys, err := fetchWikiFirmwareKeysFromAPI(cfg, proxy, insecure)
+	if err != nil {
+		return nil, err
+	}
+
+	// Save to cache
+	if cache != nil {
+		if cache.Entries == nil {
+			cache.Entries = make(map[string]FirmwareKeysCacheEntry)
+		}
+		cache.Entries[cacheKey] = FirmwareKeysCacheEntry{
+			CachedAt: time.Now(),
+			Keys:     convertWikiKeysToCache(keys),
+		}
+		if err := saveCache(cache); err != nil {
+			log.Warnf("failed to save cache: %v", err)
+		} else {
+			log.Debugf("saved keys to cache for %s %s", cfg.Device, cfg.Build)
+		}
+	}
+
+	return keys, nil
+}
+
+// fetchWikiFirmwareKeysFromAPI performs the actual API call to fetch firmware keys
+func fetchWikiFirmwareKeysFromAPI(cfg *WikiConfig, proxy string, insecure bool) (WikiFWKeys, error) {
 	var keys WikiFWKeys
 
 	client := &http.Client{
