@@ -15,11 +15,11 @@ import (
 	"github.com/apex/log"
 	"github.com/blacktop/go-macho"
 	fwcmd "github.com/blacktop/ipsw/internal/commands/fw"
-	icmd "github.com/blacktop/ipsw/internal/commands/img4"
 	"github.com/blacktop/ipsw/internal/magic"
 	"github.com/blacktop/ipsw/internal/utils"
 	"github.com/blacktop/ipsw/pkg/aea"
 	"github.com/blacktop/ipsw/pkg/ftab"
+	"github.com/blacktop/ipsw/pkg/img4"
 	"github.com/blacktop/ipsw/pkg/info"
 )
 
@@ -272,12 +272,17 @@ func ForEachIm4pInIPSW(ipswPath string, handler func(string, *macho.File) error)
 		return fmt.Errorf("failed to unzip im4p: %v", err)
 	}
 
-	for _, im4p := range im4ps {
-		if err := icmd.ExtractPayload(im4p, im4p, false); err != nil {
-			return fmt.Errorf("failed to extract im4p payload: %v", err)
-		}
-		if regexp.MustCompile(`armfw_.*.im4p$`).MatchString(im4p) {
-			ftab, err := ftab.Open(im4p)
+	for _, im4pFile := range im4ps {
+		if regexp.MustCompile(`armfw_.*.im4p$`).MatchString(im4pFile) {
+			im4p, err := img4.OpenPayload(im4pFile)
+			if err != nil {
+				return fmt.Errorf("failed to open im4p file %s: %v", im4pFile, err)
+			}
+			im4pData, err := im4p.GetData()
+			if err != nil {
+				return fmt.Errorf("failed to get data from im4p file %s: %v", im4pFile, err)
+			}
+			ftab, err := ftab.Parse(bytes.NewReader(im4pData))
 			if err != nil {
 				return fmt.Errorf("failed to parse ftab: %v", err)
 			}
@@ -294,8 +299,16 @@ func ForEachIm4pInIPSW(ipswPath string, handler func(string, *macho.File) error)
 				}
 			}
 			ftab.Close()
-		} else if regexp.MustCompile(`.*exclavecore_bundle.*im4p$`).MatchString(im4p) {
-			out, err := fwcmd.ExtractExclaveCores(im4p, os.TempDir())
+		} else if regexp.MustCompile(`.*exclavecore_bundle.*im4p$`).MatchString(im4pFile) {
+			im4p, err := img4.OpenPayload(im4pFile)
+			if err != nil {
+				return fmt.Errorf("failed to open im4p file %s: %v", im4pFile, err)
+			}
+			data, err := im4p.GetData()
+			if err != nil {
+				return fmt.Errorf("failed to get data from im4p file %s: %v", im4pFile, err)
+			}
+			out, err := fwcmd.ExtractExclaveCores(data, os.TempDir())
 			if err != nil {
 				return fmt.Errorf("failed to split exclave apps FW: %v", err)
 			}
@@ -308,11 +321,21 @@ func ForEachIm4pInIPSW(ipswPath string, handler func(string, *macho.File) error)
 				}
 			}
 		} else {
-			if m, err := macho.Open(im4p); err == nil {
-				if err := handler(filepath.Base(im4p), m); err != nil {
-					return fmt.Errorf("failed to handle macho %s: %v", im4p, err)
+			im4p, err := img4.OpenPayload(im4pFile)
+			if err != nil {
+				return fmt.Errorf("failed to open im4p file %s: %v", im4pFile, err)
+			}
+			data, err := im4p.GetData()
+			if err != nil {
+				return fmt.Errorf("failed to get data from im4p file %s: %v", im4pFile, err)
+			}
+			if m, err := macho.NewFile(bytes.NewReader(data)); err == nil {
+				if err := handler(filepath.Base(im4pFile), m); err != nil {
+					return fmt.Errorf("failed to handle macho %s: %v", im4pFile, err)
 				}
 				m.Close()
+			} else {
+				log.Debugf("failed to parse %s data as macho: %v", im4pFile, err)
 			}
 		}
 	}
