@@ -172,7 +172,8 @@ func (d *DyldDisass) Triage() error {
 			(instruction.Operation == disassemble.ARM64_ADD ||
 				instruction.Operation == disassemble.ARM64_LDR ||
 				instruction.Operation == disassemble.ARM64_LDRB ||
-				instruction.Operation == disassemble.ARM64_LDRSW) {
+				instruction.Operation == disassemble.ARM64_LDRSW ||
+				instruction.Operation == disassemble.ARM64_STRB) {
 			adrpRegister := prevInstr.Operands[0].Registers[0]
 			adrpImm := prevInstr.Operands[1].Immediate
 			if instruction.Operation == disassemble.ARM64_LDR && adrpRegister == instruction.Operands[1].Registers[0] {
@@ -183,6 +184,8 @@ func (d *DyldDisass) Triage() error {
 				adrpImm += instruction.Operands[2].Immediate
 			} else if instruction.Operation == disassemble.ARM64_LDRSW && adrpRegister == instruction.Operands[1].Registers[0] {
 				adrpImm += instruction.Operands[1].Immediate
+			} else if instruction.Operation == disassemble.ARM64_STRB && adrpRegister == instruction.Operands[1].Registers[0] {
+				adrpImm += instruction.Operands[1].Immediate
 			}
 			d.tr.Addresses[instruction.Address] = adrpImm
 		}
@@ -358,6 +361,13 @@ func (d DyldDisass) FindSymbol(addr uint64) (string, bool) {
 	return "", false
 }
 
+func (d DyldDisass) FindSwiftString(addr uint64) (string, bool) {
+	if str, ok := d.f.AddressToSymbol[addr]; ok {
+		return str, true
+	}
+	return "", false
+}
+
 func (d DyldDisass) GetCString(addr uint64) (string, error) {
 	return d.f.GetCString(addr)
 }
